/*
 * This file is part of ccsp.
 * Copyright (C) 2014  Guillaume Quintin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _CCSP_H
#define _CCSP_H

/* for size_t */
#ifdef __cplusplus
#include <cstddef>
#else
#include <stddef.h>
#endif

/* ccsp string pool */
struct ccsp_pool_bucket_struct {
  char *data;
  size_t count,hash;
  struct ccsp_pool_struct *p;
  struct ccsp_pool_bucket_struct *prev,*next;
  void *value;
};

struct ccsp_pool_struct {
  size_t n;
  size_t (*hash)(const char*,size_t);
  struct ccsp_pool_bucket_struct **tab;
};

typedef struct ccsp_pool_struct ccsp_pool[1];
typedef struct ccsp_pool_struct *ccsp_pool_ptr;

/* ccsp string pointer */
struct ccsp_string_struct {
  struct ccsp_pool_bucket_struct *b;
};

typedef struct ccsp_string_struct ccsp_string[1];
typedef struct ccsp_string_struct *ccsp_string_ptr;

/* ccsp string buffer */
struct ccsp_strbuf {
  size_t mem,len;
  char *data;
};

typedef struct ccsp_strbuf ccsp_strbuf[1];
typedef struct ccsp_strbuf *ccsp_strbuf_ptr;

/* function prototypes */
#ifdef __cplusplus
extern "C" {
#endif

extern void ccsp_set_memory_functions(void *(*)(size_t),
                                      void *(*)(void*,size_t),
                                      void (*)(void*));
void ccsp_strbuf_empty(ccsp_strbuf);
void ccsp_strbuf_init2(ccsp_strbuf,size_t);
void ccsp_strbuf_init(ccsp_strbuf);
void ccsp_strbuf_inits(ccsp_strbuf_ptr,...);
void ccsp_strbuf_clear(ccsp_strbuf);
void ccsp_strbuf_clears(ccsp_strbuf_ptr,...);
size_t ccsp_strbuf_len(ccsp_strbuf);
void ccsp_strbuf_append(ccsp_strbuf,char);
char *ccsp_strbuf_get_cstring(ccsp_strbuf);
void ccsp_string_init(ccsp_string);
void ccsp_string_inits(ccsp_string_ptr,...);
void ccsp_string_clear(ccsp_string);
void ccsp_string_clears(ccsp_string_ptr,...);
void ccsp_string_copy(ccsp_string,ccsp_string);
void ccsp_pool_init2(ccsp_pool,size_t,size_t (*)(const char*,size_t));
void ccsp_pool_init(ccsp_pool);
void ccsp_pool_inits(ccsp_pool_ptr,...);
void ccsp_pool_clear(ccsp_pool);
void ccsp_pool_clears(ccsp_pool_ptr,...);
void ccsp_intern_strbuf(ccsp_string,ccsp_pool,ccsp_strbuf);
void ccsp_intern_cstring(ccsp_string,ccsp_pool,const char*);
void ccsp_intern_cstring_alias(ccsp_string,ccsp_pool,char*);
void ccsp_string_set_value(ccsp_string,void*);
void *ccsp_string_get_value(ccsp_string);
char *ccsp_string_get_cstring(ccsp_string);

#ifdef __cplusplus
}
#endif

#endif
